package de.uni_mannheim.informatik.dws.ontmatching.kgtrackevaltwo;

import de.uni_mannheim.informatik.dws.ontmatching.yetanotheralignmentapi.Mapping;
import de.uni_mannheim.informatik.dws.ontmatching.yetanotheralignmentapi.MappingCell;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class ConfusionMatrix {
    private int truePositive;
    private int falsePositive;
    private int falseNegative;
    
    private double precision;
    private double recall;
    private double fmeasure;
    
    private double systemSize;
    private double runtime;
    
    private boolean returnedResults;
    
    public ConfusionMatrix(int tp, int fp, int fn, double prec, double rec, double fmeas, double systemSize, double runtime, boolean returnedResults){
        this.truePositive = tp;
        this.falsePositive = fp;
        this.falseNegative = fn;
        this.precision = prec;
        this.recall = rec;
        this.fmeasure = fmeas;
        this.systemSize = systemSize;
        this.runtime = runtime;
        this.returnedResults = returnedResults;
    }
    
    public ConfusionMatrix(boolean returnedResults, int runtime, Mapping ref, Mapping system){
        this(returnedResults, runtime, ref, system, true, true);
    }
    
    public ConfusionMatrix(boolean returnedResults, double runtime, Mapping ref, Mapping system, boolean kbOneDuplicateFree, boolean kbTwoDuplicateFree){
        Map<String, Set<String>> systemSourceTarget = new HashMap<>();
        Map<String, Set<String>> systemTargetSource = new HashMap<>();
        for(MappingCell systemCell : system){
            addToMap(systemSourceTarget, systemCell.getEntityOne(), systemCell.getEntityTwo());
            addToMap(systemTargetSource, systemCell.getEntityTwo(), systemCell.getEntityOne());
        }
        
        this.truePositive = 0;
        this.falsePositive = 0;
        this.falseNegative = 0;
        this.systemSize = (double) system.size();
        this.runtime = runtime;
        this.returnedResults = returnedResults;
        for(MappingCell refCell : ref){
            if(refCell.getEntityTwo().equals("null")){
                this.falsePositive += systemSourceTarget.getOrDefault(refCell.getEntityOne(), new HashSet<>()).size();
            }
            else if(refCell.getEntityOne().equals("null")){
                this.falsePositive += systemTargetSource.getOrDefault(refCell.getEntityTwo(), new HashSet<>()).size();
            }
            else{
                Set<String> systemTargets = systemSourceTarget.getOrDefault(refCell.getEntityOne(), new HashSet<>());
                Set<String> systemSources = systemTargetSource.getOrDefault(refCell.getEntityTwo(), new HashSet<>());

                if(systemTargets.contains(refCell.getEntityTwo())){
                    this.truePositive++;
                    if(kbOneDuplicateFree)
                        this.falsePositive+=systemSources.size() - 1;
                    if(kbTwoDuplicateFree)
                        this.falsePositive+=systemTargets.size() - 1;
                    
                }else{
                    this.falseNegative++;
                    if(kbOneDuplicateFree)
                        this.falsePositive+=systemSources.size();
                    if(kbTwoDuplicateFree)
                        this.falsePositive+=systemTargets.size();
                }
            }
        }
        computeDerived();
    }
    
    private static void addToMap(Map<String, Set<String>> map, String one, String two){
        Set<String> s = map.get(one);
        if(s == null){
            map.put(one, new HashSet<>(Arrays.asList(two)));
        }else{
            s.add(two);
        }
    }
    
    private void computeDerived(){
        this.precision = Util.divideWithTwoDenominators(truePositive, truePositive, falsePositive);
        this.recall = Util.divideWithTwoDenominators(truePositive, truePositive, falseNegative);
        this.fmeasure = Util.divideWithTwoDenominators((2.0 * recall * precision), recall, precision);
    }
    
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("TP:");sb.append(truePositive);
        sb.append(" FP:");sb.append(falsePositive);
        sb.append(" FN:");sb.append(falseNegative);
        sb.append(" size:");sb.append(systemSize);
        sb.append(" runtime:");sb.append(runtime);
        return sb.toString();
    }

    public int getTruePositive() {return truePositive;}
    public int getFalsePositive() {return falsePositive;}
    public int getFalseNegative() {return falseNegative;}
    public double getPrecision() {return precision;}
    public double getRecall() {return recall;}
    public double getFmeasure() {return fmeasure;}
    public double getSystemSize() {return systemSize;}
    public double getRuntime() {return runtime;}
    public boolean isReturnedResults() { return returnedResults;}    
}
